@@ -4,6 +4,7 @@ import asyncio.sslproto
 import json
 import pathlib
 import logging
+from datetime import datetime
 
 import aiofiles
 import click
@@ -18,7 +19,13 @@ from ..decorators import (
     pass_client,
     pass_session
 )
-from ..exceptions import DirectoryDoesNotExists, NotDownloadableAsAAX
+from ..exceptions import (
+    AudibleCliException,
+    DirectoryDoesNotExists,
+    DownloadUrlExpired,
+    NotDownloadableAsAAX,
+    VoucherNeedRefresh
+)
 from ..models import Library
 from ..utils import Downloader
 
@@ -28,6 +35,13 @@ logger = logging.getLogger("audible_cli.cmds.cmd_download")
 CLIENT_HEADERS = {
     "User-Agent": "Audible/671 CFNetwork/1240.0.4 Darwin/20.6.0"
 }
+datetime_type = click.DateTime([
+    "%Y-%m-%d",
+    "%Y-%m-%dT%H:%M:%S",
+    "%Y-%m-%d %H:%M:%S",
+    "%Y-%m-%dT%H:%M:%S.%fZ",
+    "%Y-%m-%dT%H:%M:%SZ"
+])
 
 
 class DownloadCounter:
@@ -40,6 +54,8 @@ class DownloadCounter:
         self._pdf: int = 0
         self._voucher: int = 0
         self._voucher_saved: int = 0
+        self._aycl = 0
+        self._aycl_voucher = 0
 
     @property
     def aax(self):
@@ -57,6 +73,24 @@ class DownloadCounter:
         self._aaxc += 1
         logger.debug(f"Currently downloaded aaxc files: {self.aaxc}")
 
+    @property
+    def aycl(self):
+        return self._aycl
+
+    def count_aycl(self):
+        self._aycl += 1
+        # log as error to display this message in any cases
+        logger.debug(f"Currently downloaded aycl files: {self.aycl}")
+
+    @property
+    def aycl_voucher(self):
+        return self._aycl_voucher
+
+    def count_aycl_voucher(self):
+        self._aycl_voucher += 1
+        # log as error to display this message in any cases
+        logger.debug(f"Currently downloaded aycl voucher files: {self.aycl_voucher}")
+
     @property
     def annotation(self):
         return self._annotation
@@ -114,7 +148,9 @@ class DownloadCounter:
             "cover": self.cover,
             "pdf": self.pdf,
             "voucher": self.voucher,
-            "voucher_saved": self.voucher_saved
+            "voucher_saved": self.voucher_saved,
+            "aycl": self.aycl,
+            "aycl_voucher": self.aycl_voucher
         }
 
     def has_downloads(self):
@@ -256,6 +292,56 @@ async def download_aax(
         counter.count_aax()
 
 
+async def _reuse_voucher(lr_file, item):
+    logger.info(f"Loading data from voucher file {lr_file}.")
+    async with aiofiles.open(lr_file, "r") as f:
+        lr = await f.read()
+    lr = json.loads(lr)
+    content_license = lr["content_license"]
+
+    assert content_license["status_code"] == "Granted", "License not granted"
+
+    # try to get the user id
+    user_id = None
+    if item._client is not None:
+        auth = item._client.auth
+        if auth.customer_info is not None:
+            user_id = auth.customer_info.get("user_id")
+
+    # Verification of allowed user
+    if user_id is None:
+        logger.debug("No user id found. Skip user verification.")
+    else:
+        if "allowed_users" in content_license:
+            allowed_users = content_license["allowed_users"]
+            if allowed_users and user_id not in allowed_users:
+                # Don't proceed here to prevent overwriting voucher file
+                msg = f"The current user is not entitled to use the voucher {lr_file}."
+                raise AudibleCliException(msg)
+        else:
+            logger.debug(f"{lr_file} does not contain allowed users key.")
+
+    # Verification of voucher validity
+    if "refresh_date" in content_license:
+        refresh_date = content_license["refresh_date"]
+        refresh_date = datetime_type.convert(refresh_date, None, None)
+        if refresh_date < datetime.utcnow():
+            raise VoucherNeedRefresh(lr_file)
+
+    content_metadata = content_license["content_metadata"]
+    url = httpx.URL(content_metadata["content_url"]["offline_url"])
+    codec = content_metadata["content_reference"]["content_format"]
+
+    expires = url.params.get("Expires")
+    if expires:
+        expires = datetime.utcfromtimestamp(int(expires))
+        now = datetime.utcnow()
+        if expires < now:
+            raise DownloadUrlExpired(lr_file)
+
+    return lr, url, codec
+
+
 async def download_aaxc(
         client, output_dir, base_filename, item,
         quality, overwrite_existing
@@ -269,7 +355,7 @@ async def download_aaxc(
             filepath = pathlib.Path(
                 output_dir) / f"{base_filename}-{codec}.aaxc"
             lr_file = filepath.with_suffix(".voucher")
-        
+
             if lr_file.is_file():
                 if filepath.is_file():
                     logger.info(
@@ -279,23 +365,23 @@ async def download_aaxc(
                         f"File {filepath} already exists. Skip download."
                     )
                     return
-                else:
-                    logger.info(
-                        f"Loading data from voucher file {lr_file}."
-                    )
-                    async with aiofiles.open(lr_file, "r") as f:
-                        lr = await f.read()
-                    lr = json.loads(lr)
-                    content_metadata = lr["content_license"][
-                        "content_metadata"]
-                    url = httpx.URL(
-                        content_metadata["content_url"]["offline_url"])
-                    codec = content_metadata["content_reference"][
-                        "content_format"]
 
-    if url is None or codec is None or lr is None:
+                try:
+                    lr, url, codec = await _reuse_voucher(lr_file, item)
+                except DownloadUrlExpired:
+                    logger.debug(f"Download url in {lr_file} is expired. Refreshing license.")
+                    overwrite_existing = True
+                except VoucherNeedRefresh:
+                    logger.debug(f"Refresh date for voucher {lr_file} reached. Refreshing license.")
+                    overwrite_existing = True
+
+    is_aycl = item.benefit_id == "AYCL"
+
+    if lr is None or url is None or codec is None:
         url, codec, lr = await item.get_aaxc_url(quality)
         counter.count_voucher()
+        if is_aycl:
+            counter.count_aycl_voucher()
 
     if codec.lower() == "mpeg":
         ext = "mp3"
@@ -331,16 +417,19 @@ async def download_aaxc(
 
     if downloaded:
         counter.count_aaxc()
+        if is_aycl:
+            counter.count_aycl()
 
 
-async def consume(queue):
+async def consume(queue, ignore_errors):
     while True:
         item = await queue.get()
         try:
             await item
         except Exception as e:
             logger.error(e)
-            raise
+            if not ignore_errors:
+                raise
         finally:
             queue.task_done()
 
@@ -443,6 +532,8 @@ def display_counter():
 
             if k == "voucher_saved":
                 k = "voucher"
+            elif k == "aycl_voucher":
+                k = "aycl voucher"
             elif k == "voucher":
                 diff = v - counter.voucher_saved
                 if diff > 0:
@@ -524,6 +615,18 @@ def display_counter():
     is_flag=True,
     help="saves the annotations (e.g. bookmarks, notes) as JSON file"
 )
+@click.option(
+    "--start-date",
+    type=datetime_type,
+    default="1970-1-1",
+    help="Only considers books added to library on or after this UTC date."
+)
+@click.option(
+    "--end-date",
+    type=datetime_type,
+    default=datetime.utcnow(),
+    help="Only considers books added to library on or before this UTC date."
+)
 @click.option(
     "--no-confirm", "-y",
     is_flag=True,
@@ -587,7 +690,9 @@ async def cli(session, api_client, **params):
     aax_fallback = params.get("aax_fallback")
     if aax_fallback:
         if get_aax:
-            logger.info("Using --aax is redundant and can be left when using --aax-fallback")
+            logger.info(
+                "Using --aax is redundant and can be left when using --aax-fallback"
+            )
         get_aax = True
         if get_aaxc:
             logger.warning("Do not mix --aaxc with --aax-fallback option.")
@@ -612,6 +717,17 @@ async def cli(session, api_client, **params):
     ignore_podcasts = params.get("ignore_podcasts")
     bunch_size = session.params.get("bunch_size")
 
+    start_date = params.get("start_date")
+    purchased_after = start_date.strftime('%Y-%m-%dT%H:%M:%S.%fZ')
+    end_date = params.get("end_date")
+    if start_date > end_date:
+        logger.error("start date must be before or equal the end date")
+        raise click.Abort()
+
+    logger.info(f"Selected start date: {purchased_after}")
+    logger.info(
+        f"Selected end   date: {end_date.strftime('%Y-%m-%dT%H:%M:%S.%fZ')}")
+
     filename_mode = params.get("filename_mode")
     if filename_mode == "config":
         filename_mode = session.config.get_profile_option(
@@ -625,7 +741,9 @@ async def cli(session, api_client, **params):
         response_groups=(
             "product_desc, media, product_attrs, relationships, "
             "series, customer_rights, pdf_url"
-        )
+        ),
+        purchased_after=purchased_after,
+        status="Active",
     )
 
     if resolve_podcats:
@@ -672,7 +790,7 @@ async def cli(session, api_client, **params):
                 ).unsafe_ask_async()
                 if answer is not None:
                     [jobs.append(i) for i in answer]
-                
+
         else:
             logger.error(
                 f"Skip title {title}: Not found in library"
@@ -699,28 +817,32 @@ async def cli(session, api_client, **params):
                 odir.mkdir(parents=True)
 
         for item in items:
-            queue_job(
-                queue=queue,
-                get_cover=get_cover,
-                get_pdf=get_pdf,
-                get_annotation=get_annotation,
-                get_chapters=get_chapters,
-                get_aax=get_aax,
-                get_aaxc=get_aaxc,
-                client=client,
-                output_dir=odir,
-                filename_mode=filename_mode,
-                item=item,
-                cover_size=cover_size,
-                quality=quality,
-                overwrite_existing=overwrite_existing,
-                aax_fallback=aax_fallback
+            purchase_date = datetime_type.convert(
+                item.purchase_date, None, None
             )
+            if start_date <= purchase_date <= end_date:
+                queue_job(
+                    queue=queue,
+                    get_cover=get_cover,
+                    get_pdf=get_pdf,
+                    get_annotation=get_annotation,
+                    get_chapters=get_chapters,
+                    get_aax=get_aax,
+                    get_aaxc=get_aaxc,
+                    client=client,
+                    output_dir=odir,
+                    filename_mode=filename_mode,
+                    item=item,
+                    cover_size=cover_size,
+                    quality=quality,
+                    overwrite_existing=overwrite_existing,
+                    aax_fallback=aax_fallback
+                )
 
     try:
         # schedule the consumer
         consumers = [
-            asyncio.ensure_future(consume(queue)) for _ in range(sim_jobs)
+            asyncio.ensure_future(consume(queue, ignore_errors)) for _ in range(sim_jobs)
         ]
         # wait until the consumer has processed all items
         await queue.join()
@@ -729,6 +851,6 @@ async def cli(session, api_client, **params):
         # the consumer is still awaiting an item, cancel it
         for consumer in consumers:
             consumer.cancel()
-    
+
         await asyncio.gather(*consumers, return_exceptions=True)
-        display_counter()
+        display_counter()
\ No newline at end of file
