@@ -1,7 +1,9 @@
 import asyncio
 import logging
+import secrets
 import string
 import unicodedata
+from datetime import datetime
 from math import ceil
 from typing import List, Optional, Union
 
@@ -11,7 +13,13 @@ from audible.aescipher import decrypt_voucher_from_licenserequest
 from audible.client import convert_response_content
 
 from .constants import CODEC_HIGH_QUALITY, CODEC_NORMAL_QUALITY
-from .exceptions import AudibleCliException, NotDownloadableAsAAX
+from .exceptions import (
+    AudibleCliException,
+    LicenseDenied,
+    NoDownloadUrl,
+    NotDownloadableAsAAX,
+    ItemNotPublished
+)
 from .utils import full_response_callback, LongestSubString
 
 
@@ -109,6 +117,9 @@ class BaseItem:
             return images[res]
 
     def get_pdf_url(self):
+        if not self.is_published():
+            raise ItemNotPublished(self.asin, self.publication_datetime)
+
         if self.pdf_url is not None:
             domain = self._client.auth.locale.domain
             return f"https://www.audible.{domain}/companion-file/{self.asin}"
@@ -119,6 +130,14 @@ class BaseItem:
                     or self.content_type == "Podcast") and self.has_children:
                 return True
 
+    def is_published(self):
+        if self.publication_datetime is not None:
+            pub_date = datetime.strptime(
+                self.publication_datetime, "%Y-%m-%dT%H:%M:%SZ"
+            )
+            now = datetime.utcnow()
+            return now > pub_date
+
 
 class LibraryItem(BaseItem):
     def _prepare_data(self, data: dict) -> dict:
@@ -216,6 +235,9 @@ class LibraryItem(BaseItem):
             return False
 
     async def get_aax_url_old(self, quality: str = "high"):
+        if not self.is_published():
+            raise ItemNotPublished(self.asin, self.publication_datetime)
+
         if not self.is_downloadable():
             raise AudibleCliException(
                 f"{self.full_title} is not downloadable."
@@ -252,6 +274,8 @@ class LibraryItem(BaseItem):
         return httpx.URL(link), codec_name
 
     async def get_aax_url(self, quality: str = "high"):
+        if not self.is_published():
+            raise ItemNotPublished(self.asin, self.publication_datetime)
 
         if not self.is_downloadable():
             raise AudibleCliException(
@@ -277,6 +301,9 @@ class LibraryItem(BaseItem):
             quality: str = "high",
             license_response_groups: Optional[str] = None
     ):
+        if not self.is_published():
+            raise ItemNotPublished(self.asin, self.publication_datetime)
+
         if not self.is_downloadable():
             raise AudibleCliException(
                 f"{self.full_title} is not downloadable."
@@ -307,12 +334,52 @@ class LibraryItem(BaseItem):
             "response_groups": response_groups
         }
 
+        headers = {
+            "X-Amzn-RequestId": secrets.token_hex(20).upper(),
+            "X-ADP-SW": "37801821",
+            "X-ADP-Transport": "WIFI",
+            "X-ADP-LTO": "120",
+            "X-Device-Type-Id": "A2CZJZGLK2JJVM",
+            "device_idiom": "phone"
+        }
         lr = await self._client.post(
             f"content/{self.asin}/licenserequest",
-            body=body
+            body=body,
+            headers=headers
         )
-        voucher = decrypt_voucher_from_licenserequest(self._client.auth, lr)
-        lr["content_license"]["license_response"] = voucher
+        content_license = lr["content_license"]
+
+        if content_license["status_code"] == "Denied":
+            if "license_denial_reasons" in content_license:
+                for reason in content_license["license_denial_reasons"]:
+                    message = reason.get("message", "UNKNOWN")
+                    rejection_reason = reason.get("rejectionReason", "UNKNOWN")
+                    validation_type = reason.get("validationType", "UNKNOWN")
+                    logger.debug(
+                        f"License denied message for {self.asin}: {message}."
+                        f"Reason: {rejection_reason}."
+                        f"Type: {validation_type}"
+                    )
+
+            msg = content_license["message"]
+            raise LicenseDenied(msg)
+
+        content_url = content_license["content_metadata"]\
+            .get("content_url", {}).get("offline_url")
+        if content_url is None:
+            raise NoDownloadUrl(self.asin)
+
+        if "license_response" in content_license:
+            try:
+                voucher = decrypt_voucher_from_licenserequest(
+                    self._client.auth, lr
+                )
+            except Exception:
+                logger.error(f"Decrypting voucher for  {self.asin} failed")
+            else:
+                content_license["license_response"] = voucher
+        else:
+            logger.error(f"No voucher for {self.asin} found")
 
         return lr
 
